/* 
 * File:   myExceptions.h
 * Author: tg
 *
 * Created on November 8, 2013, 10:16 AM
 */

#ifndef MYEXCEPTIONS_H
#define	MYEXCEPTIONS_H

#include <sstream>
#include <string>

namespace myExcepts{
    /**
     * \class myException
     * \brief generic exception
     */
    class myException {
    protected:
        const std::string msg_;
    public:
        myException (const char* text): msg_ (text) {}
        ~myException() throw() {}
        virtual const char* what() const throw() =0;
    };
    
    /**
     * \class FileMissing
     * \brief Thrown when expected file could not be opened
     */
    class FileMissing: public myException {
    public:
        FileMissing (const char* text): myException(text) {}
        const char* what() const throw () {
            std::ostringstream msg;
            msg << " *** Error: File " << msg_ 
                    << " cannot be opened\n";
            return msg.str().c_str();
        }
    };
    /**
     * \class Keyword
     * \brief Thrown when keyword not found in input file
     */
    class Keyword: public myException {
    public:
        Keyword (const char* text): myException(text) {}
        const char* what() const throw () {
            std::ostringstream msg;
            msg << " *** Error: Keyword " << msg_ 
                    << " not found\n";
            return msg.str().c_str();
        }
    };
    /**
     * \class FileHeader
     * \brief Exception to be thrown for incomplete file header
     */
    class FileHeader: public myException  {
    public:
        FileHeader (const char* text): myException (text) {}
        ~FileHeader() throw () {}
        const char*  what() const throw () {
            std::ostringstream msg;
            msg << " *** Error: Unexpected or incomplete file header. " << msg_ 
                    << " not found\n";
            return msg.str().c_str();
        }
    };
    
    /**
     * \class ImageFormat
     * \brief Exception to be thrown for unsupported image formats.
     */
    class ImageFormat: public myException {
    public:
        ImageFormat (const char* text): myException (text) {}
        ~ImageFormat() throw () {}
        const char*  what() const throw () {
            std::ostringstream msg;
            msg << " *** Error: Unsupported Image format. File " << msg_ 
                    << "\n";
            return msg.str().c_str();
        }
    };

    class DetectorType: public myException  {
    public:
        DetectorType (const char* text): myException (text) {}
        ~DetectorType() throw () {}
        const char*  what() const throw () {
            std::ostringstream msg;
            msg << " *** Error: " << msg_ 
                    << "is a time of flight machine\n";
            return msg.str().c_str();
        }
    };
    
    /**
     * \class Usage
     * \brief Thrown when program comes to stage where it should not be
     */
    class Usage: public myException {
    public:
        Usage (const char* text): myException (text) {}
        ~Usage() throw () {}
        const char*  what() const throw () {
            std::ostringstream msg;
            msg << " *** Error in Program Flow: " << msg_ 
                    << "\n";
            return msg.str().c_str();
        }
    };
}

#endif	/* MYEXCEPTIONS_H */

